package datasphere.shims.facebook;


import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.lang.reflect.Type;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.util.HashMap;
import java.util.Map;
import java.util.regex.Pattern;

import javax.crypto.Mac;
import javax.crypto.spec.SecretKeySpec;
import javax.servlet.http.Cookie;

import org.apache.commons.codec.binary.Base64;

import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;


public class CookieUtils {
	
	private CookieUtils(){}
	private static Map< String, String > cookie = null;

	/*
	 * 
	 */
	public static Map<String, String> fetch( Cookie[] cookies, String name ) {
		
		if ( cookies == null ) 
			return null;
	    
		Cookie cookie = null;		    
	    for ( Cookie c : cookies ) 
	    	if ( c.getName().equals( name ) )
	    		cookie = c;
	    
	    if ( cookie == null ) 
	    	return null;
	    
		Map< String, String > results = new HashMap< String, String >(); 
	  	String[] params = cookie.getValue().split( "&" );
    	for( String p : params ) {
	   		String[] parts = p.split( "=" );
	   		results.put( parts[ 0 ], parts[ 1 ] );
    	}
	    
	    return results;
	}
	  

	
	public static boolean  checkCookie( Cookie[] cookies, String name ) {
			
		if ( cookies == null ) 
			return false;
	    
				    
	    for ( Cookie c : cookies ){ 
	    	if ( c.getName().equals( name ) )
	    		return true;
	    }
	    return false;
	}
	
	public static String fetchValue( Cookie[] cookies, String name ) {
			
			if ( cookies == null ) 
				return null;
		    	    
		    for ( Cookie c : cookies ) 
		    	if ( c.getName().equals( name ) )
		    		return c.getValue();
		    
		    return null;
	}
	
	/**
	 * Validate the cookie 
	 */
	public static boolean validate(String value, String secret) {
        String[] splits = value.split(Pattern.quote("."));
        String signature = splits[0];
        String encoded = splits[1];
        String jsonString = decodeBase64(encoded);
        String signature2 = digest(encoded, secret);
        if (!signature2.equals(signature)) {
            return false;
        }
        //Convert the decoded json string into a cookie of type Map
        Gson gson = new Gson();
        Type type = new TypeToken<HashMap<String, String>>(){}.getType();
        cookie = gson.fromJson(jsonString, type);
        //get access token using auth code
        try {
			
			URL url = new URL(Facebook.getAuthURL(cookie.get("code")));
			HttpURLConnection connection = (HttpURLConnection) url.openConnection();
			String line;
			StringBuilder builder = new StringBuilder();
			//Make Connection
			BufferedReader reader = new BufferedReader(new InputStreamReader(connection.getInputStream()));
			while((line = reader.readLine()) != null) {
				builder.append(line);
			}
			if (builder.toString()!= null && builder.toString().contains("=")) {
			    // Split it.
				String[] parts = builder.toString().split("=");
				String key = parts[0];
				String key_value = parts[1];
				//add these values into the map
				cookie.put(key, key_value);
				
			}
			/*System.out.println("message is ************ " + builder.toString());*/
		} 
        catch (MalformedURLException e) {
			System.out.println("Malformed url " + e.getMessage());
		} 
        catch (IOException e) {
			System.out.println("IO Exception " + e.getMessage());
		} 
        
        /*for (String key : cookie.keySet()) {
            System.out.println("Key = " + key);
        }

        //iterating over values only
        for (String item : cookie.values()) {
            System.out.println("Value = " + item);
        }*/
        return true;
    }
	

	public static String decodeBase64(String encoded) {
        return new String(Base64.decodeBase64(encoded));
    }
	
	public static String digest(String encoded, String secret) {
        try {
            Mac hmacSha256 = Mac.getInstance("HmacSHA256");
            SecretKeySpec key = new SecretKeySpec(secret.getBytes("UTF-8"), "HmacSHA256");
            hmacSha256.init(key);
            byte[] mac = hmacSha256.doFinal(encoded.getBytes("UTF-8"));
            byte[] base64 = Base64.encodeBase64(mac);
            String s = new String(base64, "UTF-8");
            return s.replace("/", "_").replace("+", "-").replace("=", "");
        } catch (NoSuchAlgorithmException e) {
            throw new Error("Never thrown", e);
        } catch (UnsupportedEncodingException e) {
            throw new Error("Never thrown", e);
        } catch (InvalidKeyException e) {
            throw new RuntimeException("Invalid key", e);
        }
    }
	public static Map<String, String> getCookie() {
		return cookie;
	}

}
